<?php
/**
 * @file
 * Contains Licensing information for miniOrange SAML Login Module.
 */

namespace Drupal\miniorange_saml_idp\Form;
use Drupal\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;

class MiniorangeLicensing extends FormBase{
    
    protected RendererInterface $renderer;

    public function __construct(RendererInterface $renderer) {
        $this->renderer = $renderer;
    }

    public static function create(ContainerInterface|\Symfony\Component\DependencyInjection\ContainerInterface $container) {
        return new static(
            $container->get('renderer')
        );
    }

    public function getFormId() {
        return 'miniorange_saml_licensing';
    }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
        $base_url = \Drupal::request()->getSchemeAndHttpHost() . \Drupal::request()->getBaseUrl();

        $form['#attached']['library'] = [
            'miniorange_saml_idp/miniorange_saml_idp.admin',
            'core/drupal.dialog.ajax'
        ];

        $theme_data = [
            '#theme' => 'miniorange_saml_pricing',
            '#contact_email' => 'drupalsupport@xecurify.com',
            '#pricing_url' => 'https://portal.miniorange.com/initializepayment?requestOrigin=drupal_saml_idp_premium_plan_v8',
            '#service_providers_pricing' => [
                '1' => '$0', '2' => '$100', '3' => '$200', '4' => '$300',
                '5' => '$400', '6' => '$460', '7' => '$528', '8' => '$560',
                '9' => '$616', '10' => '$675', '15' => '$980', '20' => '$1235',
                '30' => '$1798', '40' => '$2145', '50' => '$2156',
            ],
            '#users_pricing' => [
                '1-100' => '$450', '101-200' => '$550',
                '201-300' => '$650', '301-400' => '$750',
                '401-500' => '$850', '501-1000' => '$1250',
                '1001-2000' => '$1600', '2001-3000' => '$1900',
                '3001-4000' => '$2150', '4001-5000' => '$2400',
                '5000+' => 'Contact Us'
            ],
        ];

        $rendered_theme = $this->renderer->render($theme_data);

        $form['#prefix'] = $rendered_theme;
        $cache_clear_url = Url::fromRoute('system.performance_settings')->toString();

        $rows = [
            [
                Markup::create(t('<b>1.</b> Click on <a href="#plans">Upgrade Now</a> button for required licensed plan, and you will be redirected to miniOrange login console.')),
                Markup::create(t('<b>5.</b> Uninstall and then delete the free version of the module from your Drupal site.'))
            ],
            [
                Markup::create(t('<b>2.</b> Enter your username and password with which you have created an account with us. After that you will be redirected to payment page.')),
                Markup::create(t('<b>6.</b> Now install the downloaded licensed version of the module.'))
            ],
            [
                Markup::create(t('<b>3.</b> Enter your card details and proceed for payment. On successful payment completion, the premium module(s) will be available to download.')),
                Markup::create(t('<b>7.</b> Clear Drupal Cache from <a href=":url">here</a>.', [':url' => $cache_clear_url,])),
            ],
            [
                Markup::create(t('<b>4.</b> Download the licensed module(s) from Module Releases and Downloads section.')),
                Markup::create(t('<b>8.</b> After enabling the licensed version of the module, login using the account you have registered with us. You can find the license keys <a href="https://portal.miniorange.com/viewlicense" target="_blank">here</a>.' ))
            ]
        ];

        $form['markup_how to upgrade'] = [
            '#markup' => '<h3 class="mo_saml_text_center"><br>HOW TO UPGRADE TO PAID VERSION MODULE</h3>'
        ];

        $form['miniorange_how_to_upgrade_table'] = [
            '#type' => 'table',
            '#responsive' => TRUE,
            '#rows' => $rows,
            '#attributes' => ['style' => 'border:groove', 'class' => ['mo_how_to_upgrade']],
        ];

        $form['markup_faq'] = [
            '#markup' => '<h3 class="mo_saml_faq_text_center"><br>Frequently Asked Questions</h3>'
        ];

        $form['miniorange_FAQ_table'] = [
            '#type' => 'table',
        ];

        $form['miniorange_FAQ_table'][0][0] = [
            '#type' => 'details',
            '#title' => $this->t('What is an instance?'),
            '#open' => false,
            '#attributes' => ['style' => ' width: 625px;']
        ];

        $form['miniorange_FAQ_table'][0][0]['detail1'] = [
            '#markup' => $this->t('A Drupal instance refers to a single installation of a Drupal site. It refers to each individual website where the module is active. In the case of a multisite/subsite Drupal setup, each site with a separate database will be counted as a single instance, e.g. If you have the dev-staging-prod type of environment then you will require 3 licenses of the module (with additional discounts applicable on pre-production environments).'),
        ];

        $form['miniorange_FAQ_table'][0][1] = [
            '#type' => 'details',
            '#title' => t('What is the Refund Policy?' ),
            '#open' => false,
            '#attributes' => ['style' => ' width: 625px;']
        ];

        $form['miniorange_FAQ_table'][0][1]['detail2'] = [
            '#markup' => $this->t('At miniOrange, we want to ensure 100% customer satisfaction with our product/module purchase. If the module you purchased is not working as advertised and you\'ve attempted to resolve any issues with our support team, which couldn\'t get resolved, we will refund the whole amount given that you have raised a refund request within the first 10 days of the purchase. For any queries regarding the return policy, please email us at <a href="mailto:drupalsupport@xecurify.com">drupalsupport@xecurify.com</a>.'),
        ];

        $form['miniorange_FAQ_table'][1][0] = [
            '#type' => 'details',
            '#title' => t('Does miniOrange Offer Technical Support?'),
            '#open' => false,
            '#attributes' => ['style' => 'width: 625px;']
        ];

        $form['miniorange_FAQ_table'][1][0]['detail3'] = [
            '#markup' => $this->t('Yes, we provide 24x7 support for all and any issues you might face while using the module, which includes technical support from our developers. You can get prioritized support based on the Support Plan you have opted.'),
        ];

        $form['miniorange_FAQ_table'][1][1] = [
            '#type' => 'details',
            '#title' => t('Does miniOrange store any User data ?'),
            '#open' => false,
            '#attributes' => ['style' => 'width: 625px;']
        ];

        $form['miniorange_FAQ_table'][1][1]['detail4'] = [
            '#markup' => $this->t('miniOrange does not store or transfer any user data externally. All the data remains within your premises/server.')
        ];

        return $form;
    }

    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    }

}